package com.xintong.visualinspection.dao.master;

import java.util.List;

import org.apache.ibatis.annotations.Mapper;

import com.xintong.visualinspection.bean.StatisticsBean;

/**
 * 文件名：StatisticsDao
 * 版本信息：日期：2017/3/30 Copyright 江苏省交通规划设计院 Corporation 2017 版权所有.
 */
@Mapper
public interface StatisticsDao  {
    
	public List<StatisticsBean> selectStatistics(StatisticsBean obj);
	
	public List<StatisticsBean> selectCheckedTimes(StatisticsBean obj);
	
	public List<StatisticsBean> selectFeeStationCheckNum(StatisticsBean obj);
	
	public List<StatisticsBean> selectFeeStationCheckedScore(StatisticsBean obj);
   
	public List<StatisticsBean> selectFeeStationCheckedPersonScoreDetail(StatisticsBean obj);
}
